==> makeen-expressionengine-0.0.51.tgz: package/src/functions/globals/sort.ts <==
﻿import expressionData from './../../expressionData'
import { expressionPart } from './../../compiler/parts/expressionPart'
import { sortParameterPart } from './../../compiler/parts/sortParameterPart'
import { expressionCompiler } from './../../compiler/expressionCompiler'
import { compileFlags } from './../../compiler/expressionCompiler'
import { groupDataContext } from './../../functions/groupDataContext'
import List from 'typescript-dotnet-commonjs/System/Collections/List';
import { IEnumerable } from 'typescript-dotnet-commonjs/System/Collections/Enumeration/IEnumerable'
import { mfunction } from './../mfunction'
import { Func } from 'typescript-dotnet-commonjs/System/FunctionTypes'
import Enumerable, { IOrderedEnumerable, LinqEnumerable, InfiniteLinqEnumerable } from 'typescript-dotnet-commonjs/System.Linq/Linq'
import { outputVariables } from '../../utils/outputVariables'
import IEnumerableOrArray from 'typescript-dotnet-commonjs/System/Collections/IEnumerableOrArray'
import IEnumerator from 'typescript-dotnet-commonjs/System/Collections/Enumeration/IEnumerator'
import InfiniteEnumerator from 'typescript-dotnet-commonjs/System/Collections/Enumeration/InfiniteEnumerator'

export class orderBy extends mfunction {

    public /* override */ get minParameters(): number {
        return 1;
    }

    public /* override */ get maxParameters(): number {
        return -1;
    }

    public /* internal */ /* override */ compileParameters(paramList: List<string>, flags: compileFlags): List<expressionPart> {
        let parameters: List<expressionPart> = new List<expressionPart>();
        for (let item of paramList.toArray()) {
            let spp: sortParameterPart = new sortParameterPart();
            let tokens: string[] = item.split(' ');
            if ((tokens.length > 1)) {
                spp.ascending = tokens[1].trim().startsWith("asc");//, stringComparison.OrdinalIgnoreCase);
            }
            else {
                spp.ascending = true;
            }

            spp.expression = expressionCompiler.compile(tokens[0], undefined, undefined, flags) as expressionPart;
            parameters.add(spp);
        }

        return parameters;
    }

    public invoke(expressionData: expressionData, parameters: List<expressionPart>, /* out */resolved: outputVariables, running: groupDataContext | undefined): Object | undefined {
        let obj: Object = expressionData.contexts.peek();
        let tempData: expressionData = expressionData.toSubEx();
        if (obj instanceof List) {
            let orderedResult: List<Object> | undefined = orderBy.sortBy(tempData, (<IEnumerable<Object>>(obj)), (<sortParameterPart>(parameters.get(0))), false, undefined);
            for (let i: number = 1; (i < parameters.count); i++) {
                orderedResult = orderBy.sortBy(tempData, orderedResult as IEnumerable<object>, (<sortParameterPart>(parameters.get(i))), true, undefined);
            }
            let result = new List<Object>();

            resolved.resolved = true;
            orderedResult?.forEach((o) => {
                result.add(o);
            });

            return result;
        }

        resolved.resolved = false;
        return obj;
    }

    public static sortBy(tempData: expressionData, items: IEnumerable<Object>, sortBy: sortParameterPart, thenBy: boolean, running: groupDataContext | undefined): List<Object> | undefined {
        let sortKey: expressionPart = sortBy.expression;
        let ascending: boolean = sortBy.ascending;
        try {
            const compareFunctions = (a: any, b: any): number => {
                let resolved: outputVariables = new outputVariables();
                tempData.contexts.push(a);
                let r: Object = sortKey.resolve(tempData, resolved, running) as Object;
                tempData.contexts.pop();

                resolved = new outputVariables();

                tempData.contexts.push(b);
                let r1: Object = sortKey.resolve(tempData, resolved, running) as Object;
                tempData.contexts.pop();

                if (typeof r === 'string' && typeof r1 === 'string') {
                    if (r.toLowerCase() < r1.toLowerCase()) {
                        return -1;
                    }
                    if (r.toLowerCase() > r1.toLowerCase()) {
                        return 1;
                    }
                    return 0;
                }
                else if (typeof r === 'number' && typeof r1 === 'number') {
                    return r - r1;
                }
                return 0;
                // if (r instanceof Date && r1 instanceof Date) {

                // }
            }
            const keyFuncAsc = (a: any, b: any): number => {
                return compareFunctions(a, b);
            }
            const keyFuncDesc = (a: any, b: any) => {
                return compareFunctions(b, a);
            }
            if (thenBy) {
                if (ascending) {
                    let result = ((items) as List<Object> ).sort(keyFuncAsc);
                    return result;
                }
                else {
                    let result = (items as List<Object> ).sort(keyFuncDesc);
                    return result;
                }
            }
            else if (ascending) {
                let result = (items as List<Object>).sort(keyFuncAsc);
                return result;
            }
            else {
                let result = (items as List<Object>).sort(keyFuncDesc);;
                return result;
            }

        }
        catch (e) {
            let d = e
        }
    }
}

==> makeen-expressionengine-0.0.63.tgz: package/src/functions/globals/sort.ts <==
﻿import expressionData from './../../expressionData'
import { expressionPart } from './../../compiler/parts/expressionPart'
import { sortParameterPart } from './../../compiler/parts/sortParameterPart'
import { expressionCompiler } from './../../compiler/expressionCompiler'
import { compileFlags } from './../../compiler/expressionCompiler'
import { groupDataContext } from './../../functions/groupDataContext'
import List from 'typescript-dotnet-commonjs/System/Collections/List';
import { mfunction } from './../mfunction'
import { outputVariables } from '../../utils/outputVariables'

export class orderBy extends mfunction {

     /* override */ get minParameters(): number {
        return 1;
    }

     /* override */ get maxParameters(): number {
        return -1;
    }

     /* internal */ /* override */ compileParameters(paramList: List<string>, flags: compileFlags): expressionPart[] {
        let parameters: expressionPart[] = [];
        for (let item of paramList.toArray()) {
            let spp: sortParameterPart = new sortParameterPart();
            let tokens: string[] = item.split(' ');
            if ((tokens.length > 1)) {
                spp.ascending = tokens[1].trim().startsWith("asc");//, stringComparison.OrdinalIgnoreCase);
            }
            else {
                spp.ascending = true;
            }

            spp.expression = expressionCompiler.compile(tokens[0], undefined, undefined, flags) as expressionPart;
            parameters.push(spp);
        }

        return parameters;
    }

     invoke(expressionData: expressionData, parameters: expressionPart[], /* out */resolved: outputVariables, running: groupDataContext | undefined): Object | undefined {
        let obj: Object = expressionData.contexts.peek();
        let tempData: expressionData = expressionData.toSubEx();
        if (obj instanceof List) {
            let orderedResult: List<Object> | undefined = orderBy.sortBy(tempData, obj, (<sortParameterPart>(parameters[0])), false, undefined);
            for (let i: number = 1; (i < parameters.length); i++) {
                orderedResult = orderBy.sortBy(tempData, orderedResult as List<Object>, (<sortParameterPart>(parameters[i])), true, undefined);
            }
            let result = new List<Object>();

            resolved.resolved = true;
            orderedResult?.forEach((o) => {
                result.add(o);
            });

            return result;
        }

        resolved.resolved = false;
        return obj;
    }

     static sortBy(tempData: expressionData, items: List<Object>, sortBy: sortParameterPart, thenBy: boolean, running: groupDataContext | undefined): List<Object> | undefined {
        let sortKey: expressionPart = sortBy.expression;
        let ascending: boolean = sortBy.ascending;
        try {
            const compareFunctions = (a: any, b: any): number => {
                let resolved: outputVariables = new outputVariables();
                tempData.contexts.push(a);
                let r: Object = sortKey.resolve(tempData, resolved, running) as Object;
                tempData.contexts.pop();

                resolved = new outputVariables();

                tempData.contexts.push(b);
                let r1: Object = sortKey.resolve(tempData, resolved, running) as Object;
                tempData.contexts.pop();

                if (typeof r === 'string' && typeof r1 === 'string') {
                    if (r.toLowerCase() < r1.toLowerCase()) {
                        return -1;
                    }
                    if (r.toLowerCase() > r1.toLowerCase()) {
                        return 1;
                    }
                    return 0;
                }
                else if (typeof r === 'number' && typeof r1 === 'number') {
                    return r - r1;
                }
                return 0;
                // if (r instanceof Date && r1 instanceof Date) {

                // }
            }
            const keyFuncAsc = (a: any, b: any): number => {
                return compareFunctions(a, b);
            }
            const keyFuncDesc = (a: any, b: any) => {
                return compareFunctions(b, a);
            }
            if (thenBy) {
                if (ascending) {
                    let result = items.sort(keyFuncAsc);
                    return result;
                }
                else {
                    let result = items.sort(keyFuncDesc);
                    return result;
                }
            }
            else if (ascending) {
                let result = (items as List<Object>).sort(keyFuncAsc);
                return result;
            }
            else {
                let result = (items as List<Object>).sort(keyFuncDesc);;
                return result;
            }

        }
        catch (e) {
            throw e;
        }
    }
}
